
/*
 * mojtd/hugocom.c
 *
 * 2000-05-27 Fredrik Roubert <d95fr@efd.lth.se>
 * 2003-02-03 Fredrik Roubert <d95fr@efd.lth.se>
 *
 * $Id: hugocom.c,v 1.4 2003/02/06 00:33:45 roubert Exp $
 *
 */

#include <string.h>
#include <sys/types.h>

#if _WIN32
# include <winsock2.h>
# define write(s, buf, len) send(s, buf, len, 0)
# define read(s, buf, len) recv(s, buf, len, 0)
# define close(s) shutdown(s, SD_BOTH)
typedef long ssize_t;
#else
# include <unistd.h>
# include <netdb.h>
# include <sys/socket.h>
# include <arpa/inet.h>
#endif

#include "net_interface.h"
#include "hugocom.h"


static size_t
format(char *dst, const char *src, int *status)
{
	size_t d, s, l;

	if (status == NULL)
		return 0;

	if ((l = strlen(src)) < 1)
		return 0;

	for (l --; l >= 0 && (src[l] == '\r' || src[l] == '\n'); l --)
		;

	if (l == -1)
		return 0;

	for (d = 0, s = 0; d < MOJTD_MSG_LENGTH && s <= l; s ++)
		switch (src[s])
		{
		case '\r':
			if (s < l && src[s + 1] == '\n')
				s ++;
			/* fall through */

		case '\n':
			dst[d ++] = ' ';
			break;

		default:
			dst[d ++] = src[s];
		}

	if (d == 0)
		return 0;

	*status = s <= l ? HUGO_TRUNCATED : HUGO_OK;

	dst[d ++] = '\r';
	dst[d ++] = '\n';

	return d;
}


static ssize_t
readto(int fd, void *buf, size_t nmax, const char *end)
{
	size_t l;
	ssize_t n, r;

	if ((l = strlen(end)) < 1)
		return -1;

	for (n = 0; n < (ssize_t)nmax;)
	{
		if ((r = read(fd, (char *)buf + n, nmax - n)) == -1)
			return -1;

		if (r == 0)
			return 0;

		n += r;
		if (n >= (ssize_t)l
			&& memcmp((char *)buf + n - l, end, l) == 0)
			break;
	}

	return n;
}


int
hugocom(const char *msg)
{
	static const char	str_welcome[]	= NETCOM_WELCOME,
				str_ok[]	= NETCOM_OK,
				str_empty[]	= NETCOM_EMPTY,
				str_full[]	= NETCOM_FULL,
				str_error[]	= NETCOM_ERROR,
				str_banned[]	= NETCOM_BANNED;

	int			s;
	struct sockaddr_in	addr;
	struct hostent		*hent;
	char			sbuf[MOJTD_MSG_LENGTH + 2], rbuf[1024];
	int			status;
	size_t			len;

	if ((len = format(sbuf, msg, &status)) == 0)
		return HUGO_NOTEXT;

	if ((s = socket(PF_INET, SOCK_STREAM, 0)) == -1)
		return HUGO_INTERNAL;

	addr.sin_family = PF_INET;
	addr.sin_port = htons(NETCOM_PORT);

	if ((hent = gethostbyname(NETCOM_ADDRESS)) == NULL)
		return HUGO_INTERNAL;

	addr.sin_addr.s_addr = *(u_long *)(hent->h_addr_list[0]);

	if (connect(s, (struct sockaddr *)&addr, sizeof (addr)) == -1)
		return HUGO_INTERNAL;

	if (readto(s, rbuf, STRLEN(welcome), str_welcome) == STRLEN(welcome)
		&& write(s, sbuf, len) == (ssize_t)len
		&& readto(s, rbuf, sizeof (rbuf), "\r\n\r\n") != -1)
	{
		if (memcmp(rbuf, str_ok, STRLEN(ok)) == 0)
			/* fall through */;
		else if (memcmp(rbuf, str_empty, STRLEN(empty)) == 0)
			status = HUGO_EMPTY;
		else if (memcmp(rbuf, str_full, STRLEN(full)) == 0)
			status = HUGO_FULL;
		else if (memcmp(rbuf, str_error, STRLEN(error)) == 0)
			status = HUGO_ERROR;
		else if (memcmp(rbuf, str_banned, STRLEN(banned)) == 0)
			status = HUGO_BANNED;
		else
			status = HUGO_UNKNOWN;
	}
	else
		status = HUGO_INTERNAL;

	close(s);
	return status;
}

